#include "parser.h"

TreeNode Parser::Izraz() {
	TreeNode nod;

	if (verbose) printf("Izraz->");

	if (ulaz->IsBracket('(')) {
		if (verbose) printf("( Izraz\n");

		++ulaz;
		nod = Izraz();
		if (!ulaz->IsBracket(')'))
			throw ParseError("Nedostaje ')'");
		++ulaz;
	} else if (ulaz->IsOperator('-')) {
		if (verbose) printf("- Izraz\n");

		++ulaz;
		nod = TreeNode(TreeNode::TN_operator, "-");
		nod.AddNode(TreeNode(TreeNode::TN_integer, 0));
		nod.AddNode(Izraz());
	} else if (ulaz->IsOperator('~')) {
		if (verbose) printf("~ Izraz\n");

		++ulaz;
		nod = TreeNode(TreeNode::TN_operator, "^");
		nod.AddNode(TreeNode(TreeNode::TN_integer, 0xFFFFFFFF));
		nod.AddNode(Izraz());
	} else if (ulaz->IsOperator('!')) {
		if (verbose) printf("! Izraz\n");

		++ulaz;
		nod = TreeNode(TreeNode::TN_operator, "==");
		nod.AddNode(TreeNode(TreeNode::TN_integer, 0));
		nod.AddNode(Izraz());
	} else if (ulaz->IsOperator('*')) {
		if (verbose) printf("* Izraz\n");

		++ulaz;
		nod = TreeNode(TreeNode::TN_poke);
		nod.AddNode(Izraz());
	} else if (ulaz->IsKeyword("arg")) {
		if (verbose) printf("arg Izraz\n");

		++ulaz;
		nod = TreeNode(TreeNode::TN_argument);
		nod.AddNode(Izraz());
	} else if (ulaz->IsAddress()) {
		if (verbose) printf("@ Izraz\n");

		++ulaz;
		if (!ulaz->IsVar())
			throw ParseError("Adresa se moze doznati samo od varijable");
		nod = TreeNode(TreeNode::TN_varaddr, ulaz->GetData());
		++ulaz;
	} else if (ulaz->IsKeyword("call") || ulaz->IsKeyword("stdcall")) {
		if (verbose) printf("Function\n");

		const string & calltype = ulaz->GetData();
		++ulaz;
		nod = Function(calltype);
	} else if (ulaz->IsVar() || ulaz->IsConst()) {
		if (verbose) printf("%s\n", ulaz->IsVar() ? "var" : "const");

		nod = NodeFromToken(*ulaz);
		++ulaz;
	} else {
		throw ParseError("Invalidni izraz");
	}

	if (ulaz->IsOperator()) {
		if (ulaz->IsOperator('=') &&
		    nod.GetType() != TreeNode::TN_var &&
		    nod.GetType() != TreeNode::TN_poke)
			throw ParseError("Invalidna l-vrijednost");
		if (verbose) printf("Izraz->Izraz %s Izraz\n", ulaz->GetData().c_str());

		TreeNode opn = NodeFromToken(*ulaz);
		++ulaz;
		opn.AddNode(nod);
		opn.AddNode(Izraz());
		return opn;
	}

	return nod;
}

const vector<TreeNode> Parser::Argumenti() {
	vector<TreeNode> args;
ponovi:
	if (verbose) printf("Argumenti->Izraz\n");
	args.push_back(Izraz());
	if (ulaz->IsComma()) {
		if (verbose) printf(", Argumenti\n");
		++ulaz;
		goto ponovi;
	}
	return args;
}

TreeNode Parser::Function(const string & call_type) {
	if (verbose) printf("Function->var\n");

	if (!ulaz->IsVar())
		throw ParseError("Invalidno ime funkcije");

	TreeNode tn(TreeNode::TN_call, ulaz->GetData());
	tn.CallNameToType(call_type);
	++ulaz;
	if (ulaz->IsBracket('(')) {
		if (verbose) printf(" ( Argumenti )\n");

		++ulaz;
		const vector<TreeNode> tl = Argumenti();
		for (vector<TreeNode>::const_iterator i = tl.begin(); i != tl.end(); ++i)
			tn.AddNode(*i);
		if (!ulaz->IsBracket(')'))
			throw ParseError("Nedostaje ')' kod poziva funkcije " + tn.GetName());
		++ulaz;
	}

	return tn;
}

void Parser::Block(TreeNode & tn) {
	if (verbose) printf("Block->Command\n");

	if (ulaz->IsBlock('{')) {
		++ulaz;
		CommandList(tn);
		if (!ulaz->IsBlock('}'))
			throw ParseError("Nedostaje }");
		++ulaz;
	} else {
		Command(tn);
	}
}

void Parser::Petlja(TreeNode & tn) {
	const string & tip = ulaz->GetData();

	if (verbose) printf("Petlja->");

	++ulaz;
	if (tip == "while") {
		if (verbose) printf("while Izraz Block\n");

		TreeNode petlja(TreeNode::TN_while);
		petlja.AddNode(Izraz());
		Block(petlja);
		tn.AddNode(petlja);
	} else if (tip == "for") {
		if (verbose) printf("for Izraz; Izraz; Izraz Block\n");

		TreeNode petlja(TreeNode::TN_while);
		tn.AddNode(Izraz());
		if (!ulaz->IsSemicolon())
			throw ParseError("Ocekivan prvi ; for petlje");
		++ulaz;
		petlja.AddNode(Izraz());
		if (!ulaz->IsSemicolon())
			throw ParseError("Ocekivan drugi ; for petlje");
		++ulaz;
		TreeNode kraj = Izraz();
		Block(petlja);
		petlja.AddNode(kraj);
		tn.AddNode(petlja);
	} else if (tip == "if") {
		if (verbose) printf("if Izraz Block [else Block]\n");

		TreeNode petlja(TreeNode::TN_if);
		petlja.AddNode(Izraz());
		Block(petlja);
		petlja.SetValue(petlja.GetSubNodeCount() - 1);
		if (ulaz->IsKeyword("else")) {
			++ulaz;
			Block(petlja);
		}
		tn.AddNode(petlja);
	} else {
		throw ERROR("Nedefinirana petlja \"" + tip + '"');
	}
}

void Parser::Command(TreeNode & tn) {
	int extrn = 0;

	if (verbose) printf("Command->");

	if (ulaz->IsKeyword("while") || ulaz->IsKeyword("for") ||
	    ulaz->IsKeyword("if")) {
	    	if (verbose) printf("Petlja\n");

		Petlja(tn);
	} else if (ulaz->IsKeyword("function")) {
		if (verbose) printf("function var Block\n");
funkcija:
		++ulaz;
		if (!ulaz->IsVar())
			throw ParseError("Ocekivano ime funkcije");
		const string &fname = ulaz->GetData();
		++ulaz;
		TreeNode &ttn = tn.AddNode(TreeNode(TreeNode::TN_function, fname));
		ttn.SetValue(extrn);
		Block(ttn);
	} else if (ulaz->IsKeyword("return")) {
		if (verbose) printf("return\n");

		++ulaz;
		if (!ulaz->IsSemicolon()) {
			if (verbose) printf(" Izraz ;\n");

			tn.AddNode(Izraz());
		} else {
			if (verbose) printf(" ;\n");
		}

		tn.AddNode(TreeNode(TreeNode::TN_return));

		if (!ulaz->IsSemicolon())
			throw ParseError("Ocekivan tockazarez iza return");
		++ulaz;
	} else if (ulaz->IsKeyword("push")) {
		if (verbose) printf("push var ;\n");

		++ulaz;
		if (!ulaz->IsVar())
			throw ParseError("Ocekivana varijabla iza push");

		tn.AddNode(TreeNode(TreeNode::TN_push, ulaz->GetData()));

		++ulaz;
		if (!ulaz->IsSemicolon())
			throw ParseError("Ocekivan tockazarez iza push");
		++ulaz;
	} else if (ulaz->IsKeyword("extern")) {
		if (verbose) printf("extern\n");

		++ulaz;
		if (ulaz->IsKeyword("function")) {
			if (verbose) printf(" function\n"); 
			extrn = 1;
			goto funkcija;
		}
		if (!ulaz->IsVar())
			throw ParseError("Ocekivana varijabla ili funkcija iza extern");
		if (verbose) printf(" var ;\n");

		tn.AddNode(TreeNode(TreeNode::TN_extern, ulaz->GetData()));

		++ulaz;
		if (!ulaz->IsSemicolon())
			throw ParseError("Ocekivan tockazarez iza extern");
		++ulaz;
	} else {
		if (verbose) printf("Izraz\n");

		tn.AddNode(Izraz());
		if (!ulaz->IsSemicolon())
			throw ParseError("Ocekivan tockazarez");
		++ulaz;
	}
}

void Parser::CommandList(TreeNode & tn) {
	while (!ulaz->IsEnd() && !ulaz->IsBlock('}'))
		Command(tn);
}

bool Parser::operator << (const Tokenizer & tok) {
	ulaz = tok.GetStream();
	ulaz.verbose = verbose;

	tree.root.clear();
	TreeNode &f_main = tree.root.AddNode(
				TreeNode(TreeNode::TN_function, _root_func_name)
			);

	if (verbose) ulaz->Dump();

	CommandList(f_main);
	if (!ulaz->IsEnd()) {
		throw ParseError("Ocekivan kraj ulaza, ali nije dobiven");
		return false;
	}
	return true;
}

Token Parser::GetCurrentToken() const {
	return *ulaz;
}

TreeNode Parser::NodeFromToken(const Token &tok) {
	if (tok.IsNumConst())
		return TreeNode(TreeNode::TN_integer, tok.GetInt());
	if (tok.IsStrConst())
		return TreeNode(TreeNode::TN_string, tok.GetData());
	if (tok.IsVar())
		return TreeNode(TreeNode::TN_var, tok.GetData());
	if (tok.IsOperator())
		return TreeNode(TreeNode::TN_operator, tok.GetData());

	ERROR("NodeFromToken je dobio token krivog tipa");
	return TreeNode(TreeNode::TN_integer, 0);
}

bool Parser::SetRootFunctionName(const string & fn) {
	for (string::const_iterator c = fn.begin(); c != fn.end(); ++c)
		if (!isalnum(*c) && *c != '_' && *c != '@')
			return false;
	_root_func_name = fn;
	return true;
}
