#include <stdlib.h>

#include "tokenizer.h"

const char *keywords[] = {
	"while", "call", "stdcall", "for", "if", "else", "return", "arg",
	"push", "function", "extern",
	NULL
};

string Token::ReadString(FILE *f, char type) {
	string r;
	char c;

	do {
ponovi:
		c = fgetc(f);
		r += c;
		if (c == EOF) throw TokenError("Neterminiran string");
		if (c == '\\') goto ponovi;
	} while (c != type);
	r.erase(r.end()-1);
	return r;
}

bool Token::SkipSpaces(FILE *f) {
	char c;
ponovi:
	while ((c = fgetc(f)) != EOF && isspace(c))
		if (c == '\n') ++_line;
	if (c == '#') {
		while ((c = fgetc(f)) != EOF && c != '\n');
		++_line;
		if (c != EOF) goto ponovi;
	}

	if (c == EOF) return false;
	ungetc(c, f);
	return true;
}

bool Token::Read(FILE *f, int line) {
	char c;

	_line = line;
	if (!SkipSpaces(f)) return false;

	_type = T_unknown;

	c = fgetc(f);
	if (c == '"') {
		_data = ReadString(f, '"');
		_type = T_string;
	} else if (isalnum(c) || c == '_') {
		_data = c;
		while (isalnum(c = fgetc(f)) || c == '_')
			_data += c;
		if (c != EOF) ungetc(c, f);
		if (isdigit(_data[0])) {
			char *end;
			const char *num = _data.c_str();
			_integer = strtoul(num, &end, 0);
			if (end != num + _data.size()) {
				throw TokenError("Invalidan broj");
				return false;
			}
			_type = T_number;
		} else {
			_type = T_var;

			for (const char **k = keywords; *k; ++k) {
				if (_data == *k) {
					_type = T_keyword;
					break;
				}
			}
		}
	} else {
		_data = c;
		switch (c) {
			case '+':
			case '-':
			case '*':
			case '/':
			case '<':
			case '>':
			case '=':
			case '!':
			case '&':
			case '|':
			case '^':
			case '~':
				_type = T_operator;
				if (c == '<' || c == '>' || c == '=' || c == '!') {
					c = fgetc(f);
					if (c == '=')	_data += c;
					else		ungetc(c, f);
				}
				break;
			case ',':
				_type = T_comma;
				break;
			case '@':
				_type = T_address;
				break;
			case '(':
			case ')':
				_type = T_bracket;
				break;
			case '{':
			case '}':
				_type = T_block;
				break;
			case ';':
				_type = T_semicolon;
				break;
			default:
				throw TokenError(string("Invalidan znak: ") + c);
				return false;
		}
	}
	return true;
}

void Token::Dump() const {
	printf("Token: %d (\"%s\") @ %d\n", _type, _data.c_str(), _line);
}

void Tokenizer::Read(FILE *f) {
	int l = 1;
	Token t;
	while (t.Read(f, l)) {
		tokens.push_back(t);
		l = t.GetLine();
	}
	tokens.push_back(Token(Token::T_end));
}

CTokenStream Tokenizer::GetStream() const {
	return tokens.begin();
}

void Tokenizer::Dump() const {
	printf("All tokens:\n");
	for (vector<Token>::const_iterator i = tokens.begin(); i != tokens.end(); ++i) {
		i->Dump();
	}
}
