#include "tree.h"
#include "error.h"

string TreeNode::GetTypeName(Type t) {
	switch (t) {
		case TN_unknown: return "unknown";
		case TN_operator: return "operator";
		case TN_var: return "variable";
		case TN_varaddr: return "variable address";
		case TN_integer: return "ingeger";
		case TN_string: return "string";
		case TN_call: return "call";
		case TN_function: return "function";
		case TN_argument: return "argument";
		case TN_return: return "return";
		case TN_while: return "while";
		case TN_if: return "if";
		case TN_poke: return "poke";
		case TN_push: return "push";
		case TN_extern: return "extern";
	}
	return "<Nepoznat tip!!!>";
}

string TreeNode::GetTypeName() const {
	string s = GetTypeName(_type);
	if (_type == TN_call || _type == TN_function || _type == TN_extern ||
	    _type == TN_push)
		s += ' ' + _name;
	return s;
}

void TreeNode::Dump() const {
	printf("%s", GetTypeName().c_str());
	if (_type == TN_operator || _type == TN_var || _type == TN_string) {
		printf(" \"%s\"", _name.c_str());
	} else if (_type == TN_integer) {
		printf(" 0x%X", _value);
	}
	printf("\n");
}

const TreeNode & TreeNode::GetSubNode(unsigned int p) const {
	if (p >= nodes.size())
		ERROR("GetSubNode izvan broja node-ova");
	return nodes[p];
}

TreeNode::CallType TreeNode::CallNameToType(const string & cname) {
	_value = CT_cdecl;
	if (cname == "stdcall") _value = CT_stdcall;
	return (CallType)_value;
}

void Tree::_spaces(int n) {
	while (n--) putchar(' ');
}

void Tree::_Dump(const TreeNode & branch, int level) {
	for (vector<TreeNode>::const_iterator i = branch.nodes.begin(); i != branch.nodes.end(); ++i) {
		_spaces(level);
		i->Dump();
		_Dump(*i, level + 1);
	}
}

void Tree::Dump() const {
	printf("root:\n");
	_Dump(root, 1);
}
