#ifndef _TREE_H_
#define _TREE_H_

#include <string>
#include <vector>

using namespace std;


class TreeNode {
	public:
		typedef enum {
			TN_unknown,
			TN_var,
			TN_varaddr,
			TN_integer,
			TN_string,
			TN_operator,
			TN_call,
			TN_function,
			TN_return,
			TN_argument,
			TN_while,
			TN_if,
			TN_poke,
			TN_push,
			TN_extern,
		} Type;

		typedef enum {
			CT_cdecl,
			CT_stdcall,
		} CallType;

		TreeNode(Type type = TN_unknown, const string &name = string()) : _type(type), _name(name) {}
		TreeNode(Type type, int value) : _type(type), _value(value) {}

		TreeNode & AddNode(const TreeNode &n) {
			nodes.push_back(n);
			return *(nodes.end()-1);
		}

		void Dump() const;
		const string & GetName() const { return _name; }
		int GetValue() const { return _value; }
		void SetValue(int val) { _value = val; }
		Type GetType() const { return _type; }
		string GetTypeName() const;
		static string GetTypeName(Type t);
		CallType CallNameToType(const string & cname);
		const TreeNode & GetSubNode(unsigned int p) const;
		unsigned int GetSubNodeCount() const { return nodes.size(); }
		vector<TreeNode>::const_iterator begin() const { return nodes.begin(); }
		vector<TreeNode>::const_iterator end() const { return nodes.end(); }
		void clear() { nodes.clear(); }

	private:
		vector<TreeNode> nodes;
		Type _type;
		string _name;
		int _value;

	friend class Tree;
};

class Tree {
	public:
		TreeNode root;

		void Dump() const;

	private:
		static void _Dump(const TreeNode &n, int l);
		static void _spaces(int n);
};

#endif
